package com.github.polpetta.mezzotre.orm.model;

import com.github.polpetta.types.json.ChatContext;
import io.ebean.annotation.DbJsonB;
import io.ebean.annotation.Length;
import javax.annotation.Nullable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

/**
 * TgChat represents a chat with a possible Telegram user. Here information about the chat settings,
 * such as the locale to use and the {@link ChatContext} are stored
 *
 * @author Davide Polonio
 * @since 1.0
 */
@Entity
@Table(name = "telegram_chat")
public class TgChat extends Base {

  /**
   * The ID of the conversation. For chat with users, the rules of thumb is that this id is
   * positive, while if it is a group or supergroup it is negative
   */
  @Id private final Long id;

  /**
   * This element in the database indicates which step the user is with the bot interaction. It can
   * be useful to track and store chat metadata and also have a context about the conversation
   * itself. Since the metadata stored can vary there's not a solid structured. Please use {@link
   * ChatContext#getAdditionalProperties()} to retrieve for properties that are not statically
   * typed.
   */
  @DbJsonB
  @Column(columnDefinition = "jsonb not null default '{}'::jsonb")
  @NotNull
  private ChatContext chatContext;

  /** The locale to use when chatting. Defaults to {@code en-US} */
  @NotNull
  @Length(5)
  @Column(columnDefinition = "varchar(5) not null default 'en-US'")
  private String locale;

  /**
   * See {@link #TgChat( Long, ChatContext, String)}. The default locale set here is {@code en-US}
   */
  public TgChat(Long id, ChatContext chatContext) {
    this.id = id;
    this.chatContext = chatContext;
    this.locale = "en";
  }

  /**
   * Build a new Telegram Chat
   *
   * @param id the id of the chat (can be negative)
   * @param chatContext the context of the chat, where possible metadata can be stored. See {@link
   *     ChatContext}
   * @param locale a specific locale for this chat
   */
  public TgChat(Long id, @Nullable ChatContext chatContext, String locale) {
    this.id = id;
    this.chatContext = chatContext;
    this.locale = locale;
  }

  public Long getId() {
    return id;
  }

  @NotNull
  public ChatContext getChatContext() {
    return chatContext;
  }

  public void setChatContext(@NotNull ChatContext chatContext) {
    this.chatContext = chatContext;
  }

  public String getLocale() {
    return locale;
  }

  public void setLocale(String locale) {
    this.locale = locale;
  }

  @Override
  public String toString() {
    return "TgChat{"
        + "id="
        + id
        + ", chatContext="
        + chatContext
        + ", locale='"
        + locale
        + '\''
        + '}';
  }
}
