package com.github.polpetta.mezzotre.telegram.command;

import static org.junit.jupiter.api.Assertions.*;

import com.github.polpetta.mezzotre.helper.Loader;
import com.github.polpetta.mezzotre.helper.TestConfig;
import com.github.polpetta.mezzotre.i18n.LocalizedMessageFactory;
import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.github.polpetta.mezzotre.orm.model.query.QTgChat;
import com.github.polpetta.types.json.ChatContext;
import com.google.gson.Gson;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import com.pengrad.telegrambot.request.SendMessage;
import io.ebean.Database;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executors;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.runtime.RuntimeConstants;
import org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testcontainers.containers.PostgreSQLContainer;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

@Tag("slow")
@Tag("database")
@Tag("velocity")
@Testcontainers
class StartIntegrationTest {

  private static Gson gson;

  @Container
  private final PostgreSQLContainer<?> postgresServer =
      new PostgreSQLContainer<>(TestConfig.POSTGRES_DOCKER_IMAGE);

  private VelocityEngine velocityEngine;
  private LocalizedMessageFactory localizedMessageFactory;
  private Start start;
  private Database database;

  @BeforeAll
  static void beforeAll() {
    gson = new Gson();
  }

  @BeforeEach
  void setUp() throws Exception {
    database =
        Loader.connectToDatabase(Loader.loadDefaultEbeanConfigWithPostgresSettings(postgresServer));
    velocityEngine = new VelocityEngine();
    velocityEngine.setProperty(RuntimeConstants.RESOURCE_LOADERS, "classpath");
    velocityEngine.setProperty(
        "resource.loader.classpath.class", ClasspathResourceLoader.class.getName());
    velocityEngine.init();
    localizedMessageFactory = new LocalizedMessageFactory(velocityEngine);

    final Logger log = LoggerFactory.getLogger(Start.class);

    start = new Start(localizedMessageFactory, Executors.newSingleThreadExecutor(), log);
  }

  @Test
  void shouldUpdateContextInTheDatabase() throws Exception {
    final TgChat tgChat = new TgChat(1111111L, new ChatContext());
    tgChat.setLocale("en-US");
    tgChat.save();

    final Update update =
        gson.fromJson(
            "{\n"
                + "\"update_id\":10000,\n"
                + "\"message\":{\n"
                + "  \"date\":1441645532,\n"
                + "  \"chat\":{\n"
                + "     \"last_name\":\"Test Lastname\",\n"
                + "     \"id\":1111111,\n"
                + "     \"type\": \"private\",\n"
                + "     \"first_name\":\"Test Firstname\",\n"
                + "     \"username\":\"Testusername\"\n"
                + "  },\n"
                + "  \"message_id\":1365,\n"
                + "  \"from\":{\n"
                + "     \"last_name\":\"Test Lastname\",\n"
                + "     \"id\":1111111,\n"
                + "     \"first_name\":\"Test Firstname\",\n"
                + "     \"username\":\"Testusername\"\n"
                + "  },\n"
                + "  \"text\":\"/start\"\n"
                + "}\n"
                + "}",
            Update.class);

    final CompletableFuture<Optional<BaseRequest<?, ?>>> gotFuture = start.process(tgChat, update);
    assertDoesNotThrow(() -> gotFuture.get());
    final Optional<BaseRequest<?, ?>> gotMessageOptional = gotFuture.get();
    assertDoesNotThrow(gotMessageOptional::get);
    final BaseRequest<?, ?> gotMessage = gotMessageOptional.get();
    assertInstanceOf(SendMessage.class, gotMessage);
    final String message = (String) gotMessage.getParameters().get("text");
    assertEquals(
        "**Hello Test Firstname! \uD83D\uDC4B**\n\n"
            + "This is _Mezzotre_, a simple bot focused on DnD content management! Please start by"
            + " choosing a language down below. \uD83D\uDC47",
        message);
    assertEquals(1111111L, (Long) gotMessage.getParameters().get("chat_id"));

    final TgChat retrievedTgChat = new QTgChat().id.eq(1111111L).findOne();
    assertNotNull(retrievedTgChat);
    final ChatContext gotChatContext = retrievedTgChat.getChatContext();
    assertEquals(1441645532, gotChatContext.getPreviousMessageUnixTimestampInSeconds());
    assertEquals(1365, gotChatContext.getLastMessageSentId());
    assertEquals("/start", gotChatContext.getStage());
    assertEquals(0, gotChatContext.getStep());
  }
}
